/**
 * JaDOrT: JASMINe Deployment Orchestration Tool
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JadortServiceImplTest.java 2952 2009-01-09 15:11:13Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.jadort.service.implementation;

import java.io.File;
import java.io.Serializable;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.jms.BytesMessage;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.StreamMessage;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicSubscriber;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.FlushModeType;
import javax.persistence.LockModeType;
import javax.persistence.Query;
import javax.persistence.TemporalType;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.ow2.jasmine.jadort.api.IJadortService;
import org.ow2.jasmine.jadort.api.JadortServiceException;
import org.ow2.jasmine.jadort.api.entities.deployment.OperationStateBean;
import org.ow2.jasmine.jadort.api.entities.deployment.ServerProgressBean;
import org.ow2.jasmine.jadort.api.entities.deployment.WorkerProgressBean;
import org.ow2.jasmine.jadort.api.entities.deployment.OperationStateBean.ActionState;
import org.ow2.jasmine.jadort.api.entities.deployment.OperationStateBean.Step;
import org.ow2.jasmine.jadort.api.entities.deployment.ServerProgressBean.ServerProgressState;
import org.ow2.jasmine.jadort.api.entities.deployment.WorkerProgressBean.WorkerProgressState;
import org.ow2.jasmine.jadort.api.entities.topology.GroupBean;
import org.ow2.jasmine.jadort.api.entities.topology.ServerBean;
import org.ow2.jasmine.jadort.service.action.DummyServerAction;
import org.ow2.jasmine.jadort.service.action.DummyWorkerAction;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

/**
 * Tests the JaDOrT service implementation.
 * 
 * @author Malek Chahine
 * @author Remy Bresson
 * @author S. Ali Tokmen
 */
public class JadortServiceImplTest {

    /**
     * If you want to test the whole JaDOrT service for real, set this variable
     * to true. Note that before launching this test in that mode, you must
     * deploy the JaDOrT service on a Java EE server.
     */
    private static final boolean inJ2EEServer = false;

    /**
     * Timeout (in seconds) for each serverProgress test.
     */
    private static final int deployTimeout = 100;

    /**
     * Check period (in milliseconds).
     */
    private static final int checkPeriod = 1;

    private IJadortService jadortService;

    // These need to be static since the test class is recreated at each
    // test but we'd like to keep a state log between tests
    private static String operationName;

    private static OperationStateBean operationBeanForVersion1;

    private static OperationStateBean operationBeanForVersion2;

    private static OperationStateBean operationBeanForMaintainServer1;

    private static OperationStateBean operationBeanForMaintainServer2;

    private static OperationStateBean operationBeanForMaintainNoClusterServer;

    private static IJadortService connectToJadortService() throws Exception {
        IJadortService result;
        if (JadortServiceImplTest.inJ2EEServer) {
            Hashtable<String, Object> env = new Hashtable<String, Object>();
            env.put(Context.INITIAL_CONTEXT_FACTORY, org.ow2.carol.jndi.spi.MultiOrbInitialContextFactory.class.getName());
            Context context = new InitialContext(env);
            result = (IJadortService) context.lookup("ow2.jasmine.jadort.ServiceSFB");
        } else {
            JadortServiceImpl.DUMMY = true;
            DummyServerAction.MAX_SLEEP = 1;
            DummyWorkerAction.MAX_SLEEP = 1;
            JadortServiceImpl jadortService = new JadortServiceImpl();
            jadortService.em = new DummyEntityManager();
            jadortService.session = new DummyQueueSession(jadortService.em);
            result = jadortService;
        }
        Assert.assertNull(result.getCurrentOperation());
        return result;
    }

    /**
     * @throws java.lang.Exception
     */
    @BeforeClass
    public static void staticSetUp() throws Exception {
        JadortServiceImplTest.operationName = "JadortServiceImplTest#" + UUID.randomUUID();
    }

    /**
     * @throws java.lang.Exception
     */
    @AfterClass
    public static void staticTearDown() throws Exception {
        IJadortService jadortService = JadortServiceImplTest.connectToJadortService();

        List<OperationStateBean> ops = jadortService.getOperationsList();
        for (OperationStateBean op : ops) {
            if (op.getName().equals(JadortServiceImplTest.operationName)) {
                jadortService.deleteOperation(op);
                // Don't break: each test has created an operation and the same
                // name is used each time.
            }
        }

        JadortServiceImplTest.operationName = null;
    }

    /**
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception {
        this.jadortService = JadortServiceImplTest.connectToJadortService();
    }

    /**
     * @throws java.lang.Exception
     */
    @After
    public void tearDown() throws Exception {
        this.jadortService = null;
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#createNewOperation(String)}
     * .
     */
    @Test
    public void testCreateNewOperation() throws Exception {
        this.jadortService.createNewOperation(JadortServiceImplTest.operationName);
        Assert.assertNotNull(this.jadortService.getCurrentOperation());
        Assert.assertEquals(JadortServiceImplTest.operationName, this.jadortService.getCurrentOperation().getName());
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.INITIALIZE_TOPOLOGY);
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#selectOperation(OperationStateBean)}
     * .
     */
    @Test
    public void testSelectOperation() throws Exception {
        List<OperationStateBean> ops = this.jadortService.getOperationsList();
        for (OperationStateBean op : ops) {
            if (op.getName().equals(JadortServiceImplTest.operationName)) {
                this.jadortService.selectOperation(op);
                Assert.assertEquals(this.jadortService.getCurrentOperation().getName(), JadortServiceImplTest.operationName);
                Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.INITIALIZE_TOPOLOGY);
                return;
            }
        }

        Assert.fail("Cannot find operation named \"" + JadortServiceImplTest.operationName + "\"");
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#getCurrentOperation()}
     * .
     */
    @Test
    public void testGetCurrentOperation() throws Exception {
        this.testSelectOperation();

        this.jadortService.getCurrentOperation();
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a valid and mixed topology file.
     */
    @Test
    public void testValidTopology_manyServers() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyOK_manyServers.xml");
        this.jadortService.loadTopology(new File(topo.toURI()));
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.INITIALIZE_TOPOLOGY);

        // Don't go next, we don't have that many servers in the CI !!
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a valid topology file with managers and targets.
     */
    @Test
    public void testValidTopology_manager() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyOK_manager.xml");
        this.jadortService.loadTopology(new File(topo.toURI()));
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.INITIALIZE_TOPOLOGY);
        this.jadortService.next();

        ServerBean server = this.jadortService.getGroups().get(0).getServers().get(0);
        Assert.assertEquals(ServerBean.Type.DUMMY, server.getType());
        Assert.assertEquals("weblogic1", server.getName());
        Assert.assertNull(server.getManagerConnector());

        Assert.assertEquals(ServerBean.Type.DUMMY, server.getTarget().getType());
        Assert.assertEquals("jonas1", server.getTarget().getName());
        Assert.assertNotNull(server.getTarget().getManagerConnector());
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a valid and minimal topology file.
     */
    @Test
    public void testValidTopology_minimal() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyOK_minimal.xml");
        this.jadortService.loadTopology(new File(topo.toURI()));
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a valid and maximal topology file.
     */
    @Test
    public void testValidTopology_maximal() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyOK_maximal.xml");
        this.jadortService.loadTopology(new File(topo.toURI()));
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a valid topology file with no cluster .
     */
    @Test
    public void testValidTopology_noCluster() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyOK_noCluster.xml");
        this.jadortService.loadTopology(new File(topo.toURI()));
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a topology file that's in the JASMINe Deploy ME format.
     */
    @Test
    public void testTopology_DeployME() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyOKDeployME.xml");
        this.jadortService.loadTopology(new File(topo.toURI()));
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);

        List<GroupBean> groups = this.jadortService.getGroups();
        Assert.assertEquals(1, groups.size());
        GroupBean domain = groups.get(0);
        Assert.assertFalse(domain.getClustered());
        List<ServerBean> servers = domain.getServers();
        Assert.assertEquals(2, servers.size());

        Assert.assertEquals("service:jmx:iiop://testHost/jndi/iiop://testHost:1000/iiopconnector_jonas1", servers.get(0)
            .getServerConnector().getConnectorUrl());
        Assert.assertEquals("service:jmx:rmi://otherHost/jndi/rmi://otherHost:1001/jrmpconnector_jonas2", servers.get(1)
            .getServerConnector().getConnectorUrl());
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a topology file that's in the JASMINe Deploy ME format.
     */
    @Test
    public void testTopology_DeployMEClustered() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyOKDeployME_clustered.xml");
        this.jadortService.loadTopology(new File(topo.toURI()));
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);

        List<GroupBean> groups = this.jadortService.getGroups();
        Assert.assertEquals(1, groups.size());
        GroupBean domain = groups.get(0);
        Assert.assertTrue(domain.getClustered());
        List<ServerBean> servers = domain.getServers();
        Assert.assertEquals(2, servers.size());

        Assert.assertEquals("service:jmx:iiop://testHost/jndi/iiop://testHost:1000/iiopconnector_jonas1", servers.get(0)
            .getServerConnector().getConnectorUrl());
        Assert.assertEquals("service:jmx:rmi://otherHost/jndi/rmi://otherHost:1001/jrmpconnector_jonas2", servers.get(1)
            .getServerConnector().getConnectorUrl());
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a topology file that's in the JASMINe Deploy ME format.
     */
    @Test
    public void testTopology_DeployMEWithClusterDaemon() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyOKDeployME_cd.xml");
        this.jadortService.loadTopology(new File(topo.toURI()));
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);

        List<GroupBean> groups = this.jadortService.getGroups();
        Assert.assertEquals(1, groups.size());
        GroupBean domain = groups.get(0);
        Assert.assertFalse(domain.getClustered());
        List<ServerBean> servers = domain.getServers();
        Assert.assertEquals(1, servers.size());

        Assert.assertEquals("service:jmx:rmi://localhost/jndi/rmi://localhost:1099/jrmpconnector_someRandomServerNamePrefix1",
            servers.get(0).getServerConnector().getConnectorUrl());
        Assert.assertEquals("service:jmx:iiop://localhost/jndi/iiop://localhost:2001/iiopconnector_cd1", servers.get(0)
            .getManagerConnector().getConnectorUrl());
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a topology file that's in the JASMINe Deploy ME format.
     */
    @Test
    public void testTopology_DeployMEWithClusterDaemonWithInvalidServerList() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyKODeployME_cd.xml");
        try {
            this.jadortService.loadTopology(new File(topo.toURI()));
            Assert.fail("loadTopology has accepted invalid topology!");
        } catch (JadortServiceException e) {
            Assert.assertTrue(e.getMessage().startsWith("Cannot find server"));

            // Do a call on the service to ensure it didn't die
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.INITIALIZE_TOPOLOGY);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with an invalid topology file (no server URL).
     */
    @Test
    public void testInvalidTopology_noServerURL() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyKO_noServerURL.xml");
        try {
            this.jadortService.loadTopology(new File(topo.toURI()));
            Assert.fail("loadTopology has accepted invalid topology!");
        } catch (JadortServiceException e) {
            // Do a call on the service to ensure it didn't die
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.INITIALIZE_TOPOLOGY);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with an invalid topology file (no worker URL).
     */
    @Test
    public void testInvalidTopology_noWorkerURL() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyKO_noWorkerURL.xml");
        try {
            this.jadortService.loadTopology(new File(topo.toURI()));
            Assert.fail("loadTopology has accepted invalid topology!");
        } catch (JadortServiceException e) {
            // Do a call on the service to ensure it didn't die
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.INITIALIZE_TOPOLOGY);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with an invalid topology file (wrong balancer).
     */
    @Test
    public void testInvalidTopology_wrongBalancer() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("src/test/resources/topologyKO_wrongBalancer.xml");
        try {
            this.jadortService.loadTopology(new File(topo.toURI()));
            Assert.fail("loadTopology has accepted invalid topology!");
        } catch (JadortServiceException e) {
            // Do a call on the service to ensure it didn't die
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.INITIALIZE_TOPOLOGY);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#loadTopology(File)}
     * with a non-existing topology file.
     */
    @Test
    public void testNonExistingTopology() throws Exception {
        this.testCreateNewOperation();

        File topo = new File("/dev/null");
        try {
            this.jadortService.loadTopology(new File(topo.toURI()));
            Assert.fail("loadTopology has accepted a non-existing file!");
        } catch (JadortServiceException e) {
            // Do a call on the service to ensure it didn't die
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.INITIALIZE_TOPOLOGY);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#selectGroup(GroupBean)}
     * with a migration.
     */
    @Test
    public void testSelectGroupMigrate() throws Exception {
        this.testValidTopology_maximal();

        Assert.assertEquals(1, this.jadortService.getGroups().size());

        GroupBean toSelect = this.jadortService.getGroups().get(0);
        this.jadortService.selectGroup(toSelect, IJadortService.OperationType.MIGRATE);

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_APPLICATION);
        Assert.assertEquals(toSelect.getId(), this.jadortService.getSelectedGroup().getId());
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#createApplication(java.net.URL)}
     * with an invalid application file.
     */
    @Test
    public void testCreateInvalidApplication() throws Exception {
        this.testSelectGroupMigrate();

        try {
            this.jadortService.createApplication(new File("noApplication.war").toURL());
            Assert.fail("createApplication has accepted invalid application!");
        } catch (JadortServiceException e) {
            // Do a call on the service to ensure it didn't die
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_APPLICATION);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#createApplication(java.net.URL)}
     * with a non-existing application file.
     */
    @Test
    public void testCreateNonExistingApplication() throws Exception {
        this.testSelectGroupMigrate();

        try {
            this.jadortService.createApplication(new File("/dev/null").toURL());
            Assert.fail("createApplication has accepted invalid application!");
        } catch (JadortServiceException e) {
            // Do a call on the service to ensure it didn't die
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_APPLICATION);
        }
    }

    /**
     * Integration test for application migration (forward).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testDeployVersion1() throws Exception {
        this.deployApplication("src/test/resources/application1v1/application1.ear", false);
        JadortServiceImplTest.operationBeanForVersion1 = this.jadortService.getCurrentOperation();
    }

    /**
     * Integration test for application migration (forward).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testDeployVersion2() throws Exception {
        this.deployApplication("src/test/resources/application1v2/application1.ear", false);
        JadortServiceImplTest.operationBeanForVersion2 = this.jadortService.getCurrentOperation();
    }

    /**
     * Integration test for application migration (backwards).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testUndeployVersion2() throws Exception {
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForVersion2);
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForVersion2.getId());

        this.jadortService.selectOperation(JadortServiceImplTest.operationBeanForVersion2);
        Assert.assertNotNull(this.jadortService.getCurrentOperation());

        this.undeployApplication();
    }

    /**
     * Integration test for application migration (backwards).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testUndeployVersion1() throws Exception {
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForVersion1);
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForVersion1.getId());

        this.jadortService.selectOperation(JadortServiceImplTest.operationBeanForVersion1);
        Assert.assertNotNull(this.jadortService.getCurrentOperation());

        this.undeployApplication();
    }

    /**
     * Integration test for application migration (forward, complete).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testRedeployVersion1() throws Exception {
        this.deployApplication("src/test/resources/application1v1/application1.ear", true);
    }

    /**
     * Integration test for application migration (forward, complete).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testRedeployVersion2() throws Exception {
        this.deployApplication("src/test/resources/application1v2/application1.ear", true);
    }

    private void deployApplication(final String applicationPath, final boolean eraseOld) throws Exception {
        this.testSelectGroupMigrate();

        this.jadortService.createApplication(new File(applicationPath).toURI().toURL());

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_APPLICATION);
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 0);

        // Upload application
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.UPLOAD_OK, 33);

        // Deploy application
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.DEPLOY_OK, 66);

        // Set application as default
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.SET_DEFAULT_OK, 100);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);

        if (this.jadortService.getCurrentOperation().getServerProgressList().get(0).getOldApplication() != null) {
            // Change page
            this.jadortService.next();
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART2);
            this.waitForFinishAndCheckServerProgressList(ServerProgressState.SET_DEFAULT_OK, 0);

            // Undeploy old version
            this.jadortService.next();
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART2);
            this.waitForFinishAndCheckServerProgressList(ServerProgressState.UNDEPLOY_OK, 50);

            if (eraseOld) {
                // Erase old version
                this.jadortService.next();
                Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART2);
                this.waitForFinishAndCheckServerProgressList(ServerProgressState.ERASE_OK, 100);

                // Change page
                Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART2);
                this.jadortService.next();
                Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.FINISHED);
            }
        } else {
            // Change page
            this.jadortService.next();
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.FINISHED);
        }
    }

    private void undeployApplication() throws Exception {
        if (this.jadortService.getCurrentOperation().getServerProgressList().get(0).getOldApplication() != null) {
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART2);
            this.waitForFinishAndCheckServerProgressList(ServerProgressState.UNDEPLOY_OK, 50);

            // Redeploy old version
            this.jadortService.previous();
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART2);
            this.waitForFinishAndCheckServerProgressList(ServerProgressState.SET_DEFAULT_OK, 0);
        }

        // Change page
        this.jadortService.previous();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.SET_DEFAULT_OK, 100);

        // Set old version as default
        this.jadortService.previous();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.DEPLOY_OK, 66);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);

        // Undeploy new version
        this.jadortService.previous();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.UPLOAD_OK, 33);

        // Erase new version
        this.jadortService.previous();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 0);

        // Change page
        this.jadortService.previous();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_APPLICATION);
    }

    private void waitForFinishAndCheckServerProgressList(final ServerProgressState state, final int progress) throws Exception {
        boolean notFinished = true;
        while (notFinished) {
            notFinished = false;
            for (ServerProgressBean dep : this.jadortService.getServerProgressList()) {
                if (dep.getActionState().equals(ActionState.FINISHED_ERROR)) {
                    throw new Exception("ServerProgressListBean " + dep + " has errors. Log:\n" + dep.getLog());
                } else if (dep.getActionState().equals(ActionState.WAITING)
                    || dep.getActionState().equals(ActionState.FINISHED_OK)) {
                    notFinished = notFinished || false;
                } else {
                    notFinished = notFinished || true;
                }
            }
            if (!notFinished) {
                Thread.sleep(JadortServiceImplTest.checkPeriod);
            }
        }
        for (ServerProgressBean dep : this.jadortService.getServerProgressList()) {
            Assert.assertEquals(dep.getActionState(), ActionState.WAITING);
            Assert.assertEquals(dep.getServerProgressState(), state);
            Assert.assertEquals(dep.getProgress(), progress);
        }
    }

    /**
     * Test method for
     * {@link org.ow2.jasmine.jadort.service.implementation.JadortServiceImpl#selectGroup(GroupBean)}
     * with a maintenance.
     */
    @Test
    public void testSelectGroupMaintenanceAndUndo() throws Exception {
        this.selectGroupMaintenance(true, true);
    }

    /**
     * Select group with mode "maintenance".
     */
    private void selectGroupMaintenance(final boolean undo, final boolean clustered) throws Exception {
        if (clustered) {
            this.testValidTopology_maximal();
        } else {
            this.testValidTopology_noCluster();
        }

        Assert.assertEquals(1, this.jadortService.getGroups().size());

        GroupBean toSelect = this.jadortService.getGroups().get(0);
        this.jadortService.selectGroup(toSelect, IJadortService.OperationType.MAINTAIN);

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        Assert.assertEquals(toSelect.getId(), this.jadortService.getSelectedGroup().getId());

        if (undo) {
            this.jadortService.previous();
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);
        }
    }

    /**
     * Integration test for server maintenance (all servers maintained in one
     * step) (forward).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testMaintainClusterAllServers() throws Exception {
        this.selectGroupMaintenance(false, true);
        JadortServiceImplTest.operationBeanForMaintainServer1 = this.jadortService.getCurrentOperation();

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        List<ServerBean> serversToSelect = new ArrayList<ServerBean>(this.jadortService.getSelectedGroup().getServers());
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.FINISHED);
    }

    /**
     * Integration test for no clustered servers maintenance (all servers
     * maintained in one step) (forward).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testMaintainNoClusterAllServers() throws Exception {
        this.selectGroupMaintenance(false, false);
        JadortServiceImplTest.operationBeanForMaintainNoClusterServer = this.jadortService.getCurrentOperation();

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        List<ServerBean> serversToSelect = new ArrayList<ServerBean>(this.jadortService.getSelectedGroup().getServers());
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainNoClusterServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.FINISHED);
    }

    /**
     * Integration test for server maintenance, try to maintain the same server
     * twice to have an error.
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testAttemptToMaintainTwice() throws Exception {
        this.selectGroupMaintenance(false, true);
        JadortServiceImplTest.operationBeanForMaintainServer2 = this.jadortService.getCurrentOperation();

        // Maintain the first server
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        List<ServerBean> serversToSelect = new ArrayList<ServerBean>();
        serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(0));
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);

        // Try to maintain the first server again
        serversToSelect = new ArrayList<ServerBean>();
        serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(0));
        try {
            this.jadortService.selectServers(serversToSelect);
            Assert.fail();
        } catch (JadortServiceException ignored) {
            this.unMaintainServers(false);
        }
    }

    /**
     * Integration test for server maintenance, try to select servers with
     * different progress percentiles to have an error.
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testSelectDifferentProgress() throws Exception {
        this.selectGroupMaintenance(false, true);
        JadortServiceImplTest.operationBeanForMaintainServer2 = this.jadortService.getCurrentOperation();

        // Maintain the first server
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        List<ServerBean> serversToSelect = new ArrayList<ServerBean>();
        serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(0));
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);

        // Try to maintain all servers, including the one maintained before
        serversToSelect = new ArrayList<ServerBean>();
        for (int i = 0; i < this.jadortService.getSelectedGroup().getServers().size(); i++) {
            serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(i));
        }
        try {
            this.jadortService.selectServers(serversToSelect);
            Assert.fail();
        } catch (JadortServiceException ignored) {
            this.unMaintainServers(false);
        }
    }

    /**
     * Integration test for server maintenance (servers maintained in two
     * passes) (forward).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testMaintainClusterServersInTwoSteps() throws Exception {
        this.selectGroupMaintenance(false, true);
        JadortServiceImplTest.operationBeanForMaintainServer2 = this.jadortService.getCurrentOperation();

        // Maintain the first server
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        List<ServerBean> serversToSelect = new ArrayList<ServerBean>();
        serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(0));
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);

        // Maintain the other servers
        serversToSelect = new ArrayList<ServerBean>();
        for (int i = 1; i < this.jadortService.getSelectedGroup().getServers().size(); i++) {
            serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(i));
        }
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.FINISHED);
    }

    /**
     * Integration test for server maintenance No cluster (servers maintained in
     * two passes) (forward).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testMaintainNoClusterServersInTwoSteps() throws Exception {
        this.selectGroupMaintenance(false, false);

        // Maintain the first server
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        List<ServerBean> serversToSelect = new ArrayList<ServerBean>();
        serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(0));
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainNoClusterServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);

        // Maintain the other servers
        serversToSelect = new ArrayList<ServerBean>();
        for (int i = 1; i < this.jadortService.getSelectedGroup().getServers().size(); i++) {
            serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(i));
        }
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainNoClusterServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.FINISHED);
    }

    private void maintainServers() throws Exception {
        // In our test topology file we have 2 workers per server
        Assert.assertEquals(2 * this.jadortService.getCurrentOperation().getServerProgressList().size(), this.jadortService
            .getCurrentOperation().getWorkerProgressList().size());

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 0);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.INITIAL, 0);

        // Stop Worker
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 20);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 20);

        // Stop Server
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.STOP_OK, 40);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 40);

        // Maintain Server
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.MAINTAIN_OK, 60);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 60);

        // Stop Server
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.START_OK, 80);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 80);

        // Stop Worker
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.START_OK, 100);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.START_OK, 100);
    }

    private void maintainNoClusterServers() throws Exception {
        // In our test topology file we have 2 workers per server
        Assert.assertEquals(2 * this.jadortService.getCurrentOperation().getServerProgressList().size(), this.jadortService
            .getCurrentOperation().getWorkerProgressList().size());

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 0);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.INITIAL, 0);

        // Stop Worker
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.DISABLE_APPLICATIONS_OK, 16);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.INITIAL, 16);

        // Stop Server
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.DISABLE_APPLICATIONS_OK, 33);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 33);

        // Maintain Server
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.STOP_OK, 50);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 50);

        // Maintain Server
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.MAINTAIN_OK, 66);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 66);

        // Stop Server
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.START_OK, 83);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 83);

        // Stop Worker
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.START_OK, 100);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.START_OK, 100);
    }

    /**
     * Integration test for server maintenance (backwards).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testUnmaintainClusterServers1() throws Exception {
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForMaintainServer1);
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForMaintainServer1.getId());

        this.jadortService.selectOperation(JadortServiceImplTest.operationBeanForMaintainServer1);
        Assert.assertNotNull(this.jadortService.getCurrentOperation());

        this.unMaintainServers(true);
    }

    /**
     * Integration test for server maintenance (backwards).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testUnmaintainClusterServers2() throws Exception {
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForMaintainServer2);
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForMaintainServer2.getId());

        this.jadortService.selectOperation(JadortServiceImplTest.operationBeanForMaintainServer2);
        Assert.assertNotNull(this.jadortService.getCurrentOperation());

        this.unMaintainServers(true);
    }

    /**
     * Integration test for no cluster servers maintenance (backwards).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testUnmaintainNoClusterServers() throws Exception {
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForMaintainNoClusterServer);
        Assert.assertNotNull(JadortServiceImplTest.operationBeanForMaintainNoClusterServer.getId());

        this.jadortService.selectOperation(JadortServiceImplTest.operationBeanForMaintainNoClusterServer);
        Assert.assertNotNull(this.jadortService.getCurrentOperation());

        this.unMaintainNoClusterServers(true);
    }

    /**
     * Integration test for server maintenance (some of the servers maintained,
     * then everything undone) (forward).
     */
    @Test(timeout = JadortServiceImplTest.deployTimeout * 1000)
    public void testMaintainSomeServersThenUnmaintain() throws Exception {
        this.selectGroupMaintenance(false, true);

        // Maintain the first server
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        List<ServerBean> serversToSelect = new ArrayList<ServerBean>();
        serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(0));
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);

        // Maintain the second server
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        serversToSelect = new ArrayList<ServerBean>();
        serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(1));
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        this.maintainServers();

        // Change page
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);

        // Start maintaining the third server and undo in the middle
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        serversToSelect = new ArrayList<ServerBean>();
        serversToSelect.add(this.jadortService.getSelectedGroup().getServers().get(2));
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 0);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.INITIAL, 0);

        // Stop Worker
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 20);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 20);

        // Undo the third server
        this.jadortService.previous();
        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 0);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.INITIAL, 0);

        // Keep on undoing
        this.jadortService.previous();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        this.unMaintainServers(false);
    }

    private void unMaintainServers(final boolean finished) throws Exception {
        if (finished) {
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.FINISHED);
        }

        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.START_OK, 100);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.START_OK, 100);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);

        // Stop Server
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.START_OK, 80);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 80);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);

        // Stop Worker
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.MAINTAIN_OK, 60);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 60);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);

        // Maintain Server
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.STOP_OK, 40);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 40);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);

        // Start Server
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 20);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 20);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);

        // Start Worker
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 0);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.INITIAL, 0);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);

        // Previous screen is either SELECT_SERVERS or SELECT_GROUP
        this.jadortService.previous();

        if (this.jadortService.getCurrentOperation().getCurrentStep().equals(Step.SELECT_SERVERS)) {
            this.jadortService.previous();
        }

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);
    }

    private void unMaintainNoClusterServers(final boolean finished) throws Exception {
        if (finished) {
            Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.FINISHED);
        }

        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.START_OK, 100);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.START_OK, 100);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);

        // Stop Server
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.START_OK, 83);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 83);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);

        // Stop Worker
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.MAINTAIN_OK, 66);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 66);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);

        // Maintain Server
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.STOP_OK, 50);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 50);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);

        // Maintain Server
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.DISABLE_APPLICATIONS_OK, 33);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.STOP_OK, 33);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);

        // Start Worker
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.DISABLE_APPLICATIONS_OK, 16);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.INITIAL, 16);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);

        // Enable Applications
        this.jadortService.previous();
        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 0);
        this.waitForFinishAndCheckWorkerProgressList(WorkerProgressState.INITIAL, 0);
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_NO_CLUSTER);

        // Previous screen is either SELECT_SERVERS or SELECT_GROUP
        this.jadortService.previous();

        if (this.jadortService.getCurrentOperation().getCurrentStep().equals(Step.SELECT_SERVERS)) {
            this.jadortService.previous();
        }

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);
    }

    private void waitForFinishAndCheckWorkerProgressList(final WorkerProgressState state, final int progress) throws Exception {
        boolean notFinished = true;
        while (notFinished) {
            notFinished = false;
            for (WorkerProgressBean dep : this.jadortService.getWorkerProgressList()) {
                if (dep.getActionState().equals(ActionState.FINISHED_ERROR)) {
                    throw new Exception("WorkerProgressListBean " + dep + " has errors. Log:\n" + dep.getLog());
                } else if (dep.getActionState().equals(ActionState.WAITING)
                    || dep.getActionState().equals(ActionState.FINISHED_OK)) {
                    notFinished = notFinished || false;
                } else {
                    notFinished = notFinished || true;
                }
            }
            if (!notFinished) {
                Thread.sleep(JadortServiceImplTest.checkPeriod);
            }
        }
        for (WorkerProgressBean dep : this.jadortService.getWorkerProgressList()) {
            Assert.assertEquals(dep.getActionState(), ActionState.WAITING);
            Assert.assertEquals(dep.getWorkerProgressState(), state);
            Assert.assertEquals(dep.getProgress(), progress);
        }
    }

    /**
     * Integration test for the delete operation's behavior with migration
     * actions that are in progress.
     */
    @Test
    public void testOperationDeletionNotAllowedWhenMigrating() throws Exception {
        this.testSelectGroupMigrate();

        this.jadortService.createApplication(new File("src/test/resources/application1v1/application1.ear").toURI().toURL());

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_APPLICATION);
        this.jadortService.next();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MIGRATION_PART1);

        // Save current operation and call setUp for reconnection
        OperationStateBean current = this.jadortService.getCurrentOperation();
        this.setUp();
        try {
            this.jadortService.deleteOperation(current);
            Assert.fail("Managed to delete an operation that's migrating!");
        } catch (JadortServiceException ignored) {
            // Expected exception
        }

        // Go to previous step to enable deletion
        Assert.assertNull(this.jadortService.getCurrentOperation());
        this.jadortService.selectOperation(current);
        this.jadortService.previous();
    }

    /**
     * Integration test for the delete operation's behavior with maintenance
     * actions are in progress.
     */
    @Test
    public void testOperationDeletionNotAllowedWhenMaintaining() throws Exception {
        this.selectGroupMaintenance(false, true);

        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_SERVERS);
        List<ServerBean> serversToSelect = new ArrayList<ServerBean>(this.jadortService.getSelectedGroup().getServers());
        this.jadortService.selectServers(serversToSelect);
        this.jadortService.next();

        Assert.assertEquals(serversToSelect, this.jadortService.getSelectedServers());
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.EXECUTING_MAINTENANCE_CLUSTER);

        // Save current operation and call setUp for reconnection
        OperationStateBean current = this.jadortService.getCurrentOperation();
        this.setUp();
        try {
            this.jadortService.deleteOperation(current);
            Assert.fail("Managed to delete an operation that's maintaining!");
        } catch (JadortServiceException ignored) {
            // Expected exception
        }

        // Go to previous step twice to enable deletion
        Assert.assertNull(this.jadortService.getCurrentOperation());
        this.jadortService.selectOperation(current);
        this.jadortService.previous();
        this.jadortService.previous();
        Assert.assertEquals(this.jadortService.getCurrentOperation().getCurrentStep(), Step.SELECT_GROUP);
    }

    protected static class DummyEntityManager implements EntityManager {
        protected static int lastId = 0;

        protected static Map<Integer, Object> contents = new HashMap<Integer, Object>();

        public void clear() {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void close() {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public boolean contains(final Object arg0) {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public Query createNamedQuery(final String arg0) {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public Query createNativeQuery(final String arg0) {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        @SuppressWarnings("unchecked")
        public Query createNativeQuery(final String arg0, final Class arg1) {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public Query createNativeQuery(final String arg0, final String arg1) {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public Query createQuery(final String arg0) {
            return new DummyQuery(arg0);
        }

        @SuppressWarnings("unchecked")
        public <T> T find(final Class<T> arg0, final Object arg1) {
            Object result = DummyEntityManager.contents.get(arg1);
            if (result.getClass().equals(arg0)) {
                return (T) result;
            } else {
                return null;
            }
        }

        public void flush() {
            // Nothing
        }

        public Object getDelegate() {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public FlushModeType getFlushMode() {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public <T> T getReference(final Class<T> arg0, final Object arg1) {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public EntityTransaction getTransaction() {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public boolean isOpen() {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void joinTransaction() {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void lock(final Object arg0, final LockModeType arg1) {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public <T> T merge(final T arg0) {
            this.persist(arg0);
            return arg0;
        }

        @SuppressWarnings("unchecked")
        public void persist(final Object arg0) {
            boolean present = DummyEntityManager.contents.values().contains(arg0);
            if (!present) {
                DummyEntityManager.lastId++;
                DummyEntityManager.contents.put(DummyEntityManager.lastId, arg0);
            }
            for (Field field : arg0.getClass().getDeclaredFields()) {
                for (Annotation annotation : field.getAnnotations()) {
                    if (annotation.annotationType().equals(javax.persistence.Id.class)) {
                        if (!present) {
                            boolean set = false;
                            String setter = "set" + field.getName();
                            for (Method method : arg0.getClass().getMethods()) {
                                if (method.getName().equalsIgnoreCase(setter)) {
                                    try {
                                        if (field.getType().equals(int.class)) {
                                            method.invoke(arg0, DummyEntityManager.lastId);
                                        } else {
                                            method.invoke(arg0, Integer.valueOf(DummyEntityManager.lastId));
                                        }
                                        set = true;
                                    } catch (Exception e) {
                                        throw new IllegalArgumentException("Failed setting id for object \"" + arg0.toString()
                                            + "\"", e);
                                    }
                                }
                            }
                            if (!set) {
                                throw new IllegalArgumentException("Failed setting id field \"" + field.getName()
                                    + "\" of object \"" + arg0.toString() + "\"");
                            }
                        }
                    } else if (annotation.annotationType().equals(javax.persistence.OneToOne.class)) {
                        boolean set = false;
                        String getter = "get" + field.getName();
                        for (Method method : arg0.getClass().getMethods()) {
                            if (method.getName().equalsIgnoreCase(getter)) {
                                try {
                                    Object o = method.invoke(arg0);
                                    if (o != null) {
                                        this.persist(o);
                                    }
                                    set = true;
                                } catch (Exception e) {
                                    throw new IllegalArgumentException("Failed persisting child \"" + field.getName()
                                        + "\" of object \"" + arg0.toString() + "\"", e);
                                }
                            }
                        }
                        if (!set) {
                            throw new IllegalArgumentException("Failed setting id field \"" + field.getName()
                                + "\" of object \"" + arg0.toString() + "\"");
                        }
                    } else if (annotation.annotationType().equals(javax.persistence.OneToMany.class)) {
                        boolean set = false;
                        String getter = "get" + field.getName();
                        for (Method method : arg0.getClass().getMethods()) {
                            if (method.getName().equalsIgnoreCase(getter)) {
                                try {
                                    Collection c = (Collection) method.invoke(arg0);
                                    if (c != null) {
                                        for (Object o : c) {
                                            this.persist(o);
                                        }
                                    }
                                    set = true;
                                } catch (Exception e) {
                                    throw new IllegalArgumentException("Failed persisting child \"" + field.getName()
                                        + "\" of object \"" + arg0.toString() + "\"", e);
                                }
                            }
                        }
                        if (!set) {
                            throw new IllegalArgumentException("Failed setting id field \"" + field.getName()
                                + "\" of object \"" + arg0.toString() + "\"");
                        }
                    }
                }
            }
        }

        public void refresh(final Object arg0) {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void remove(final Object arg0) {
            for (Map.Entry<Integer, Object> entry : DummyEntityManager.contents.entrySet()) {
                if (entry.getValue().equals(arg0)) {
                    DummyEntityManager.contents.remove(entry.getKey());
                    break;
                }
            }
        }

        public void setFlushMode(final FlushModeType arg0) {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        private class DummyQuery implements Query {

            private String query;

            public DummyQuery(final String query) {
                this.query = query;
            }

            public int executeUpdate() {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            @SuppressWarnings("unchecked")
            public List getResultList() {
                List result = new ArrayList();
                for (Object o : DummyEntityManager.contents.values()) {
                    if (this.query.contains(' ' + o.getClass().getSimpleName() + ' ')
                        || this.query.contains('_' + o.getClass().getSimpleName() + ' ')) {
                        result.add(o);
                    }
                }
                return result;
            }

            public Object getSingleResult() {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setFirstResult(final int arg0) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setFlushMode(final FlushModeType arg0) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setHint(final String arg0, final Object arg1) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setMaxResults(final int arg0) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setParameter(final String arg0, final Object arg1) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setParameter(final int arg0, final Object arg1) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setParameter(final String arg0, final Date arg1, final TemporalType arg2) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setParameter(final String arg0, final Calendar arg1, final TemporalType arg2) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setParameter(final int arg0, final Date arg1, final TemporalType arg2) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Query setParameter(final int arg0, final Calendar arg1, final TemporalType arg2) {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

        }
    }

    private static class DummyQueueSession implements QueueSession {
        private EntityManager em;

        public DummyQueueSession(final EntityManager em) {
            this.em = em;
        }

        public QueueBrowser createBrowser(final Queue arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public QueueBrowser createBrowser(final Queue arg0, final String arg1) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public Queue createQueue(final String arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public QueueReceiver createReceiver(final Queue arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public QueueReceiver createReceiver(final Queue arg0, final String arg1) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public QueueSender createSender(final Queue arg0) throws JMSException {
            return new DummyQueueSender(this.em);
        }

        public TemporaryQueue createTemporaryQueue() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void close() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void commit() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public BytesMessage createBytesMessage() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public MessageConsumer createConsumer(final Destination arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public MessageConsumer createConsumer(final Destination arg0, final String arg1) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public MessageConsumer createConsumer(final Destination arg0, final String arg1, final boolean arg2)
            throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public TopicSubscriber createDurableSubscriber(final Topic arg0, final String arg1) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public TopicSubscriber createDurableSubscriber(final Topic arg0, final String arg1, final String arg2,
            final boolean arg3) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public MapMessage createMapMessage() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public Message createMessage() throws JMSException {
            // TODO Auto-generated method stub
            return new DummyMessage();
        }

        public ObjectMessage createObjectMessage() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public ObjectMessage createObjectMessage(final Serializable arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public MessageProducer createProducer(final Destination arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public StreamMessage createStreamMessage() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public TemporaryTopic createTemporaryTopic() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public TextMessage createTextMessage() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public TextMessage createTextMessage(final String arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public Topic createTopic(final String arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public int getAcknowledgeMode() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public MessageListener getMessageListener() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public boolean getTransacted() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void recover() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void rollback() throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void run() {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void setMessageListener(final MessageListener arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        public void unsubscribe(final String arg0) throws JMSException {
            // TODO Auto-generated method stub
            throw new NotImplementedException();
        }

        private class DummyQueueSender implements QueueSender {
            private EntityManager em;

            public DummyQueueSender(final EntityManager em) {
                this.em = em;
            }

            public Queue getQueue() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void send(final Message arg0) throws JMSException {
                JadortServiceJMSActions jmsAction = new JadortServiceJMSActions();
                jmsAction.em = this.em;
                jmsAction.onMessage(arg0);
            }

            public void send(final Queue arg0, final Message arg1) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void send(final Message arg0, final int arg1, final int arg2, final long arg3) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void send(final Queue arg0, final Message arg1, final int arg2, final int arg3, final long arg4)
                throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void close() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public int getDeliveryMode() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Destination getDestination() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public boolean getDisableMessageID() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public boolean getDisableMessageTimestamp() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public int getPriority() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public long getTimeToLive() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void send(final Destination arg0, final Message arg1) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void send(final Destination arg0, final Message arg1, final int arg2, final int arg3, final long arg4)
                throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setDeliveryMode(final int arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setDisableMessageID(final boolean arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setDisableMessageTimestamp(final boolean arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setPriority(final int arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setTimeToLive(final long arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }
        }

        private class DummyMessage implements Message {
            private Map<String, Boolean> boolProperties = new HashMap<String, Boolean>();

            private Map<String, Integer> intProperties = new HashMap<String, Integer>();

            private Map<String, String> stringProperties = new HashMap<String, String>();

            public void acknowledge() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void clearBody() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void clearProperties() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public boolean getBooleanProperty(final String arg0) throws JMSException {
                return this.boolProperties.get(arg0);
            }

            public byte getByteProperty(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public double getDoubleProperty(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public float getFloatProperty(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public int getIntProperty(final String arg0) throws JMSException {
                return this.intProperties.get(arg0);
            }

            public String getJMSCorrelationID() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public byte[] getJMSCorrelationIDAsBytes() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public int getJMSDeliveryMode() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Destination getJMSDestination() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public long getJMSExpiration() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public String getJMSMessageID() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public int getJMSPriority() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public boolean getJMSRedelivered() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Destination getJMSReplyTo() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public long getJMSTimestamp() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public String getJMSType() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public long getLongProperty(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public Object getObjectProperty(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            @SuppressWarnings("unchecked")
            public Enumeration getPropertyNames() throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public short getShortProperty(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public String getStringProperty(final String arg0) throws JMSException {
                return this.stringProperties.get(arg0);
            }

            public boolean propertyExists(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setBooleanProperty(final String arg0, final boolean arg1) throws JMSException {
                this.boolProperties.put(arg0, arg1);
            }

            public void setByteProperty(final String arg0, final byte arg1) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setDoubleProperty(final String arg0, final double arg1) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setFloatProperty(final String arg0, final float arg1) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setIntProperty(final String arg0, final int arg1) throws JMSException {
                this.intProperties.put(arg0, arg1);
            }

            public void setJMSCorrelationID(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSCorrelationIDAsBytes(final byte[] arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSDeliveryMode(final int arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSDestination(final Destination arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSExpiration(final long arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSMessageID(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSPriority(final int arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSRedelivered(final boolean arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSReplyTo(final Destination arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSTimestamp(final long arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setJMSType(final String arg0) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setLongProperty(final String arg0, final long arg1) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setObjectProperty(final String arg0, final Object arg1) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setShortProperty(final String arg0, final short arg1) throws JMSException {
                // TODO Auto-generated method stub
                throw new NotImplementedException();
            }

            public void setStringProperty(final String arg0, final String arg1) throws JMSException {
                this.stringProperties.put(arg0, arg1);
            }
        }
    }
}
